#pragma once

#include <functional>
#include <memory>
#include <string>
#include <vector>

namespace scene {
class BusinessServer;

using Func = std::function<int()>;

class Application {
public:
    static Application& getInstance();
    int exec(int argc, char* argv[]);
    void notifyAllConnections(const std::string& signalName, const std::string& input);
    void setFunc(Func f) { func = std::move(f); }

private:
    Application() = default;
    ~Application();
    bool parseArgv(int argc, char* argv[]);
    void init();
    void run();

    static void printHelp();
    void initConfig();
    void initLog();
    void initBusinessServer();

private:
    ::std::string confDir;
    ::std::string confFilename;
    std::shared_ptr<BusinessServer> server;
    Func func;
};
}  // namespace scene
