#pragma once

#include <string>

typedef enum : int {
    LOG_LEVEL_FATAL,
    LOG_LEVEL_ERROR,
    LOG_LEVEL_WARN,
    LOG_LEVEL_INFO,
    LOG_LEVEL_DEBUG,
    LOG_LEVEL_END
} LogLevel;

extern LogLevel LOG_LEVEL;
extern void setLogLevel(LogLevel logLevel);

#define PRINT_FATAL(fmt, ...)                               \
    do {                                                    \
        if (LOG_LEVEL >= LOG_LEVEL_FATAL) {                 \
            fprintf(stderr, "[FATAL] " fmt, ##__VA_ARGS__); \
        }                                                   \
    } while (0)

#define PRINT_ERROR(fmt, ...)                               \
    do {                                                    \
        if (LOG_LEVEL >= LOG_LEVEL_ERROR) {                 \
            fprintf(stderr, "[ERROR] " fmt, ##__VA_ARGS__); \
        }                                                   \
    } while (0)

#define PRINT_WARN(fmt, ...)                               \
    do {                                                   \
        if (LOG_LEVEL >= LOG_LEVEL_WARN) {                 \
            fprintf(stdout, "[WARN] " fmt, ##__VA_ARGS__); \
            fflush(stdout);                                \
        }                                                  \
    } while (0)

#define PRINT_INFO(fmt, ...)                               \
    do {                                                   \
        if (LOG_LEVEL >= LOG_LEVEL_INFO) {                 \
            fprintf(stdout, "[INFO] " fmt, ##__VA_ARGS__); \
            fflush(stdout);                                \
        }                                                  \
    } while (0)

#define PRINT_DEBUG(fmt, ...)                               \
    do {                                                    \
        if (LOG_LEVEL >= LOG_LEVEL_DEBUG) {                 \
            fprintf(stdout, "[DEBUG] " fmt, ##__VA_ARGS__); \
            fflush(stdout);                                 \
        }                                                   \
    } while (0)
